@@ -319,14 +319,9 @@ impl<A> State<A> {
         let start_ptr = self.tip.as_mut_ptr()
                                 .offset(self.tip.len_items() as isize);
 
-        let padding = start_ptr as usize % alignment;
-
-        debug_assert!(padding < alignment);
-        debug_assert_eq!(padding, 0);
-
+        let padding = (alignment - (start_ptr as usize % alignment)) % alignment;
         let start_ptr = start_ptr.offset(AlignedVec::<A>::bytes_to_items(padding) as isize);
-
-        let new_used = self.tip.len_items() + padding + AlignedVec::<A>::bytes_to_items(size);
+        let new_used = self.tip.len_items() + AlignedVec::<A>::bytes_to_items(padding + size);
 
         if new_used <= self.tip.capacity_items() {
             self.tip.set_len_items(new_used);
@@ -604,4 +599,18 @@ mod tests {
         // Dropping the stack itself does nothing to the drop counter of course.
         assert_eq!(drop_count.get(), 1);
     }
+
+    #[test]
+    fn test_large_alignment() {
+        // Large enough that the alignment overflows the default initial capacity
+        #[repr(align(256))]
+        #[derive(Copy, Clone)]
+        struct LargeAlign(u8);
+
+        let obstack = Obstack::new();
+        let val_ref = obstack.push_copy(LargeAlign(0));
+
+        let address = val_ref as *mut _ as usize;
+        assert!(address % std::mem::align_of::<LargeAlign>() == 0);
+    }
 }
